ETL
===

Mini ETL tool to transfer from a CSV or XLSX file to a REST API or a database.

Requirements
------------

* Python2 >= 2.7 or Python3 >= 3.4
* Gevent is required for parallel handling

Usage
-----

    $ etl URL /path/to/file.csv

### Sources

The source is a CSV file. The first row contains the headers. The dialect
(separation of the fields, the rows, text delimiters)  of the CSV are
automatically guessed.

If the file's name is **-** the file reads the standard output.

### Targets

The target is a API endpoint presented as a complete URL. An optional token may
be used to authenticate the request:

    $ etl -T mytokenstring ...
    $ etl --token mytokenstring ...

    POST /api/v1/endpoint/ HTTP/1.1
    Authorization: Token mytokenstring

A dry run option *-n*, *--dry-run* can be set to print the body of the request
in the standard output instead of posting it.

### HTTPS considerations

ETL supports http as well as https. The *-k* option used with an https target
will disable the verification of the SSL certificate.

#### Using POST and PUT

ETL will send POST request by default. The *--PUT* argument will send PUT
requests instead.

#### Using the record to build the URL

ETL will format the URL with the record when the URL contains a **{**. It will
use *str.format* to build the string.

    $ echo -n 'Foo,bar\nA,true' | etl http://localhost/{Foo}/baz/ -
    Request POST http://localhost/A/baz


### Records manipulations

ETL allows to manipulate the records. This includes, renaming, parsing the
keys.

#### Transformations

The transformations change the data types of the values. They behave like the
equivalent python functions, with the exception of ``bool`` that returns true
if the value is equal to the string "true".

    $ echo -n 'Foo,bar\nA,true' | ./venv/bin/python bin/etl -n http://localhost/  -
    {"Foo": "A", "bar": "true"}
    $ echo -n 'Foo,bar\nA,true' | ./venv/bin/python bin/etl -n http://localhost/  - -p bar=bool
    {"Foo": "A", "bar": true}

#### Renaming

The *-H* option indicates that the first line contains headers.  The names of
the columns are guessed from the headers unless there are *-c* options forcing
the names of the columns.

    $ echo -n 'Foo,bar\nA,1' | ./venv/bin/python bin/etl _  - -n 
    {"Foo": "A", "bar": "1"}
    $ echo -n 'Foo,bar\nA,1' | ./venv/bin/python bin/etl _  - -n -c col1 -c col2
    {"col2": "1", "col1": "A"}
    $ echo -n 'Foo,bar\nA,1' | ./venv/bin/python bin/etl _  - -n -H 
    ["Foo", "bar"]
    ["A", "1"]
    $ echo -n 'Foo,bar\nA,1' | ./venv/bin/python bin/etl _  - -n -c col1 -c col2 -H
    {"col2": "bar", "col1": "Foo"}
    {"col2": "1", "col1": "A"}

The renaming changes the name of the fields from the source. 

    $ echo -n 'Foo,bar\nA,1' | ./venv/bin/python bin/etl _  - -n -m bar=baz
    {"Foo": "A", "baz": "1"}
