# -*- coding: utf-8 -*-


class Backend(object):
    placeholder = u'%s'
    without_password = False

    def __repr__(self):
        return self.__class__.__name__

    def open_connection(self, **kw):
        raise NotImplementedError('Backend %s must implement build open_connection')

    def escape_name(self, name):
        return '"{}"'.format(name)

    def escape_column_name(self, table_name):
        return self.escape_name(table_name)

    def escape_table_name(self, table_name):
        return self.escape_name(table_name)

    def build_insert_query(self, table, record):
        sql_query = u'INSERT INTO {table} ({fields}) VALUES ({placeholders})'
        sql_query = sql_query.format(
            table=self.escape_table_name(table),
            fields=u', '.join(self.escape_column_name(column) for column in record.keys()),
            placeholders=u', '.join([self.placeholder] * len(record)),
        )
        return sql_query, list(record.values())
