# -*- coding: utf-8 -*-

import collections
import os.path

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

from etl.targets import PatternTarget, FixedTarget

from .http import HTTPDestination
from .virtual import DevNull
from .database import DatabaseDestination
from .file import FileDestination

DESTINATION_CLASSES = [
    HTTPDestination,
    DatabaseDestination,
    FileDestination,
]

Context = collections.namedtuple('Context', ['insecure_ssl', 'token', 'fragment', 'ext'])


def get_destination(target_url, token=None, dryrun=False, insecure_ssl=False):
    target = PatternTarget(target_url) if '{' in target_url else FixedTarget(target_url)

    parsed_url = urlparse(target_url)

    name, ext = os.path.splitext(parsed_url.path)
    ctx = Context(insecure_ssl, token, parsed_url.fragment, ext)

    for Destination in DESTINATION_CLASSES:
        dest = Destination.from_scheme(target, parsed_url.scheme, ctx)
        if dest is not None:
            break
    else:
        raise NotImplementedError('Export to {} is not supported'.format(parsed_url.scheme))
    if dryrun:
        dest = DevNull(dest)

    return dest
