# -*- coding: utf-8 -*-

import logging

from etl.databases.backends import get_backend, NoBackend
from etl.databases import DatabaseConnections

logger = logging.getLogger(__name__)


class DatabaseDestination(object):
    @classmethod
    def from_scheme(cls, target, scheme, ctx):
        try:
            backend_class = get_backend(scheme)
        except NoBackend:
            return None
        backend = backend_class()
        return cls(target, DatabaseConnections(backend, ctx.token))

    def __init__(self, target, connections):
        self.connections = connections
        self.target = target

    def __repr__(self):
        return '{}({})'.format(self.connections, self.target)

    def __enter__(self):
        return self

    def __exit__(self, exc_value, exc_type, traceback):
        self.connections.close()

    def __call__(self, record):
        target_url = self.target(record)
        prepared_request = self.connections.build_insert_query(target_url, record)
        logger.debug('Run %s', prepared_request)
        prepared_request()
