
from __future__ import absolute_import

import logging

from etl.process.simple import Process

try:
    import gevent
    from gevent import monkey as curious_george
    from gevent.queue import Queue
except ImportError:
    raise RuntimeError('Gevent is required for etl.gevent')

# Monkey-patch.
curious_george.patch_all(thread=False, select=False)

logger = logging.getLogger(__name__)


class ParallelProcess(Process):
    def __init__(self, source, destination, parallel):
        self.queue = Queue(parallel * 2)
        self.workers = [gevent.spawn(self.worker, x) for x in range(parallel)]
        self.real_destination = destination
        super(ParallelProcess, self).__init__(source, self.queue.put)

    def __repr__(self):
        return '<Process[x{}]>'.format(len(self.workers))

    def __call__(self):
        gevent.spawn(super(ParallelProcess, self).__call__).join()
        logger.debug('All the sources are loaded')
        for x in self.workers:
            self.queue.put(StopIteration)

        gevent.joinall(self.workers)

    def worker(self, n):
        logger.debug('Worker %s started', n)
        for record in self.queue:
            self.real_destination(record)
        logger.debug('Worker %s ended', n)
