# -*- coding: utf-8 -*-


import logging

logger = logging.getLogger(__name__)


class Wrap(object):
    def __init__(self, callback):
        self.callback = callback

    def __enter__(self):
        return self.callback

    def __exit__(self, exc_value, exc_type, traceback):
        return


class Process(object):
    def __init__(self, source, destination):
        self.source = source
        self.destination = destination

    def __repr__(self):
        return '<Process>'

    def __call__(self):
        logger.info('Starting process %s', self)

        destination = self.destination
        if not hasattr(destination, '__enter__'):
            destination = Wrap(destination)

        i = 0
        source = iter(self.source)
        with destination as destination_session:
            while True:
                try:
                    record = next(source)
                except StopIteration:
                    break
                except Exception:
                    logger.exception('Cannot get record %s', i)
                    continue
                i += 1
                logger.debug('Sending record %s: %s', i, record)
                try:
                    destination_session(record)
                except Exception:
                    logger.exception('Cannot send record %s', i)
        logger.info('Ending process %s, did %s records', self, i)
