# -*- coding: utf-8 -*-

import os.path
import sys

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

from .csv import CSVSource
from .json import JSONSource
from .database import DatabaseSource


def get_source(filename, content_type=None, tokens=None):
    parsed_url = urlparse(filename)
    if not parsed_url.scheme or parsed_url.scheme == 'file':
        name, ext = os.path.splitext(parsed_url.path)
        if not content_type:
            if ext == '.csv':
                content_type = 'text/csv'
            elif ext == '.json':
                content_type = 'application/json'
            elif ext == '.xlsx':
                content_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
            elif 'csv' in parsed_url.fragment:
                content_type = 'text/csv'
            elif 'json' in parsed_url.fragment:
                content_type = 'application/json'
            elif 'xslx' in parsed_url.fragment:
                content_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'

        infile = sys.stdin if parsed_url.path == '-' else open(parsed_url.path, 'rb')
        if content_type == 'application/json':
            return JSONSource(infile)
        elif content_type == 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':
            from .excel import ExcelSource
            return ExcelSource(infile)
        elif content_type == 'text/csv':
            return CSVSource(infile)
        else:
            raise ValueError('Cannot guess the mime type of {}'.format(filename))
    else:
        return DatabaseSource.build(parsed_url.scheme, filename, tokens)
