# -*- coding: utf-8 -*-

from __future__ import absolute_import

import logging
import codecs
import csv


logger = logging.getLogger(__name__)

SNIFF_SIZE = 2048


class CSVSource(object):
    def __init__(self, source):
        StreamReader = codecs.getreader('utf-8')
        reader = StreamReader(source)
        try:
            source.tell()
        except IOError:
            dialect = csv.excel
        else:
            dialect = self._sniff(reader)

        self._reader = csv.reader(reader, dialect)

    def __repr__(self):
        return '<{}>'.format(self.__class__.__name__)

    @classmethod
    def _sniff(cls, source):
        pos = source.tell()
        dialect = csv.Sniffer().sniff(source.read(SNIFF_SIZE))
        source.seek(pos)
        return dialect

    def __iter__(self):
        for row in self._reader:
            yield [col if col != '' else None for col in row]
