# -*- coding: utf-8 -*-

import logging

from etl.databases.backends import get_backend
from etl.databases import DatabaseConnections

logger = logging.getLogger(__name__)


class DatabaseSource(object):
    @classmethod
    def build(cls, backend_name, source_url, tokens, args=None):
        backend = get_backend(backend_name)
        connection = DatabaseConnections(backend(), tokens)
        return cls(source_url, connection, args=args)

    def __init__(self, source_url, connections, args=None):
        self.connections = connections
        self.source_url = source_url
        self.args = args or []

    def __repr__(self):
        return '{}({})'.format(self.connections, self.source_url)

    def __iter__(self):
        conn, query = self.connections.get_connection(self.source_url)
        if query.startswith('/') or query.startswith('.'):
            with open(query, 'r') as query_file:
                query = query_file.read()

        logger.debug('Run %s (%s)', query, self.args)
        cursor = conn.cursor()
        try:
            cursor.execute(query, self.args)
            names = [c[0] for c in cursor.description]
            for line in cursor:
                yield dict(zip(names, line))
        finally:
            cursor.close()
