# -*- coding: utf-8 -*-

import logging
import getpass

logger = logging.getLogger(__name__)


def get_token_provider(system_tokens, token):
    if system_tokens:
        return SystemTokenProvider()
    if token:
        return FixedTokenProvider(token)
    return NoToken()


class NoToken(object):
    def __call__(self, **details):
        logger.warning('Calling a credential for %s while there are no tokens', details)
        return None

    def __bool__(self):
        return False

    __nonzero__ = __bool__


class FixedTokenProvider(object):
    def __init__(self, token):
        self.token = token

    def __repr__(self):
        return self.token

    def __call__(self, **details):
        return self.token


class SystemTokenProvider(object):
    def __init__(self, keyring=None):
        if keyring is None:
            import keyring as keyring_module
            keyring = keyring_module.get_keyring()
        self.keyring = keyring

    def __call__(self, host, username, **kw):
        pw = self.keyring.get_password(host, username)
        if pw is None:
            if username:
                label = 'Password for {}@{}: '.format(username, host)
            else:
                label = 'Password for {}: '.format(host)
            pw = getpass.getpass(label)
            self.keyring.set_password(host, username, pw)
        return pw
